@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXQuatd.cpp,v 1.19 2005/01/16 16:06:07 fox Exp $                         *
+* $Id: FXQuatd.cpp,v 1.19.2.1 2005/06/22 06:15:59 fox Exp $                         *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -277,7 +277,7 @@ void FXQuatd::setAxes(const FXVec3d& ex,const FXVec3d& ey,const FXVec3d& ez){
     y=0.25*scale;
     x=(ex.y+ey.x)/scale;
     z=(ey.z+ez.y)/scale;
-    w=(ex.z-ez.x)/scale;
+    w=(ez.x-ex.z)/scale;
     }
   else{
     scale=2.0*sqrt(1.0+ez.z-ex.x-ey.y);
