@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXScrollBar.cpp,v 1.22.2.1 2006/01/13 02:54:57 fox Exp $                     *
+* $Id: FXScrollBar.cpp,v 1.22 2005/01/16 16:06:07 fox Exp $                     *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -584,9 +584,6 @@ long FXScrollBar::onMouseWheel(FXObject*,FXSelector,void* ptr){
           getApp()->addTimeout(this,ID_TIMEWHEEL,5,(void*)(FXival)dragjump);
           }
         }
-      else{
-        dragpoint=0;
-        }
       return 1;
       }
     }
@@ -609,7 +606,7 @@ long FXScrollBar::onTimeWheel(FXObject*,FXSelector,void* ptr){
       if(target) target->tryHandle(this,FXSEL(SEL_CHANGED,message),(void*)(FXival)pos);
       }
     }
-  else if(dragpoint>pos){
+  else{
     if(p>=dragpoint){
       setPosition(dragpoint);
       dragpoint=0;
@@ -621,9 +618,6 @@ long FXScrollBar::onTimeWheel(FXObject*,FXSelector,void* ptr){
       if(target) target->tryHandle(this,FXSEL(SEL_CHANGED,message),(void*)(FXival)pos);
       }
     }
-  else{
-    dragpoint=0;
-    }
   return 1;
   }
 
