@@ -410,8 +410,8 @@ void FXTopWindow::place(FXuint placement){
     GetMonitorInfoA(hMon,&minfo);
     rx=minfo.rcWork.left;
     ry=minfo.rcWork.top;
-    rw=minfo.rcWork.right;
-    rh=minfo.rcWork.bottom;
+    rw=minfo.rcWork.right-minfo.rcWork.left;
+    rh=minfo.rcWork.bottom-minfo.rcWork.top;
     }
   else
 #endif
